/* 
 * Cache manager, part of the:
 *  BSD Telephony Of Mexico "Emiliano" MIPL browser/interpreter, version 1.4  12/23/00
 *
 *   Part of the "Zapata" Computer Telephony Technology.
 *
 *   See http://www.bsdtelephony.com.mx 
 *
 *   For documentation on the MIPL language and technology
 *
 *   See http://www.mipl.org
 *
 *
 *  The technologies, software, hardware, designs, drawings, scheumatics, board
 *  layouts and/or artwork, concepts, methodologies (including the use of all
 *  of these, and that which is derived from the use of all of these), all other
 *  intellectual properties contained herein, and all intellectual property
 *  rights have been and shall continue to be expressly for the benefit of all
 *  mankind, and are perpetually placed in the public domain, and may be used,
 *  copied, and/or modified by anyone, in any manner, for any legal purpose,
 *  without restriction. 
 *
 */


#include <stdio.h>
#include <dirent.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <gdbm.h>

struct node {
	char *fname;
	time_t ftime;
	int nkbytes;
	int indb;
	int delme;
} ;

#define	MAXNAME 100000

#define	THEDIR "/usr/local/miplcache"
#define CACHEDB "/usr/local/miplcache/cache" 

struct node nodes[MAXNAME],*nodep[MAXNAME];

void *root = NULL;
int totalkbytes = 0;
int threshkbytes = 4000000; /* 4 GIG default */

int finddelim(str,strp,delimchr,quotechr)
char *str;
char *strp[];
{
int     i,inquo;

        inquo = 0;
        i = 0;
        strp[i++] = str;
        if (!*str)
           {
                strp[0] = 0;
                return(0);
           }
        for(; *str; str++)
           {
                if (*str == quotechr)
                   {
                        if (inquo)
                           {
                                *str = 0;
                                inquo = 0;    
                           }
                        else
                           {
                                strp[i - 1] = str + 1;
                                inquo = 1;
                           }
                   }
                if ((*str == delimchr) && (!inquo))
                   {
                        *str = 0;
                        strp[i++] = str + 1;
                   }
           }
        strp[i] = 0;
        return(i);

}
    

int node_compare(struct node **node1, struct node **node2)
   {
      if ((*node1)->ftime > (*node2)->ftime)
           return (-1);
      if ((*node1)->ftime < (*node2)->ftime)
           return (1);
      return (0);
   }

int nks(n)  /* return number of K bytes */
int n;
{
	if (!n) return(0);
	return(((n - 1) / 1024) + 1);
}

main(argc,argv)
int argc;
char *argv[];
{
int	i,j = 0,k;
char	a[256],*strs1[100];
DIR *dirp;
struct stat mystat;
struct dirent *dp;
long	l;
time_t	t;
struct node *nodeptr = nodes;
GDBM_FILE dbf;
datum   data,key,nextkey;  
FILE	*dfp;

	if (argc > 1) threshkbytes = atoi(argv[1]);

	dirp = opendir(THEDIR);
	if (!dirp)
	   {
		perror("opendir");
		exit(255);
	   }
	/* read-in all of the file names */
	while((dp = readdir(dirp)) && (j < MAXNAME))
	   {
		i = strlen(dp->d_name);
		  /* has to have at least 3 chars in name */
		if (i < 3) continue;
		if (strncmp(dp->d_name,"AIVR",4)) continue;
		  /* skip dottage and stuff */
		if (*dp->d_name == '.') continue; 
		sprintf(a,"%s/%s",THEDIR,dp->d_name);
		if (stat(a,&mystat) == -1) continue;
		  /* must be a regular file */
		if ((mystat.st_mode & S_IFMT) != S_IFREG) continue;
		if ((nodeptr->fname = malloc(i + 1)) == NULL)
		   {
			perror("malloc");
			exit(255);
		   }
		nodeptr->ftime = mystat.st_mtime;  /* save file mod. time */
		strcpy(nodeptr->fname,dp->d_name);
		nodeptr->nkbytes = nks(mystat.st_size);
		nodeptr->indb = nodeptr->delme = 0;
		nodep[j++] = nodeptr++;
		nodeptr++;
	   }
	closedir(dirp);
	/* sort the files most-recent first */
	qsort((char *) nodep,j,sizeof(struct node *),node_compare);
	for(i = 0; i < j; i++)
	   {
		  /* if over the threshold, zot the file */
		if (totalkbytes >= threshkbytes)
		   {
			nodep[i]->delme = 1;
			sprintf(a,"%s/%s",THEDIR,nodep[i]->fname);
			unlink(a); 
		   }
		totalkbytes += nodep[i]->nkbytes;
	   }
	printf("\nOriginally, Total K bytes: %d\n",totalkbytes);

	dfp = tmpfile();
	if (!dfp)
	   {
		perror("temp file open");
		exit(255);
	   }

	/* open up the DB */
        dbf = gdbm_open(CACHEDB,0,GDBM_WRITER,0,NULL);
        if (!dbf)
	   {
		perror("Cannot open db");
		exit(255);
	   }

	/* make sure we have filenames for all the database entries */

	key = gdbm_firstkey(dbf);
	while(key.dptr)
	   {
		data = gdbm_fetch(dbf,key);
		if (!data.dptr) 
		   {
			nextkey = gdbm_nextkey(dbf,key);
			free(key.dptr);
			key = nextkey;
			continue;
		   }
                strs1[0] = strs1[1] = strs1[2] = 0;
                if (finddelim(data.dptr,strs1,' ','\"') > 2)
		   {
			for(i = 0,k = 0; i < j; i++)
			   {
				  /* skip files to be deleted */
				if (nodep[i]->delme) continue;
				sprintf(a,"%s/%s",THEDIR,nodep[i]->fname);
				  /* bail early if found */
				if (!strcmp(a,strs1[2])) 
				   {
					nodep[i]->indb = 1;
					k = 1;
					break;
				   }
			   }
			if (!k) fprintf(dfp,"%s\n",key.dptr);
		   }
		free(data.dptr);
		nextkey = gdbm_nextkey(dbf,key);
		free(key.dptr);
		key = nextkey;
	   }
	rewind(dfp);
	/* go through and nuke all of the db entries that had no file names,
	   or at least dont anymore after the file has been deleted */
	while(fgets(a,sizeof(a) - 1,dfp))
	   {
		if (a[strlen(a) - 1] == '\n') a[strlen(a) - 1] = 0;
		key.dptr = a;
		key.dsize = strlen(a) + 1;
		gdbm_delete(dbf,key);
	   }
	fclose(dfp);
	/* go through and nuke all of the files that had no db entry */
	for(i = 0; i < j; i++)
	   {
		  /* if over the threshold, zot the file */
		if (!nodep[i]->indb)
		   {
			sprintf(a,"%s/%s",THEDIR,nodep[i]->fname);
			unlink(a); 
		   }
	   }
	exit(0);
}

