/* 
 * MIPL parser and interpreter, part of:
 *  BSD Telephony Of Mexico "Emiliano" MIPL browser/interpreter, version 1.4  12/23/00
 *
 *   Part of the "Zapata" Computer Telephony Technology.
 *
 *   See http://www.bsdtelephony.com.mx 
 *
 *   For documentation on the MIPL language and technology
 *
 *   See http://www.mipl.org
 *
 *
 *  The technologies, software, hardware, designs, drawings, scheumatics, board
 *  layouts and/or artwork, concepts, methodologies (including the use of all
 *  of these, and that which is derived from the use of all of these), all other
 *  intellectual properties contained herein, and all intellectual property
 *  rights have been and shall continue to be expressly for the benefit of all
 *  mankind, and are perpetually placed in the public domain, and may be used,
 *  copied, and/or modified by anyone, in any manner, for any legal purpose,
 *  without restriction. 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "mipl.h"
#include "zap.h"

TAG maintag;  /* not to be confused with maytag */
FORM form; /* the global form output storage */
AUTH auth; /* the authentication stuff */
char outbuf[500];
int out_mode = 0;
int vok = 0;
#ifndef	NORECORD
char rectemp[500];
#endif
extern FILE *readurl();
extern ZAP *zap;

int determine_text(mode,gotvoice)
int mode,gotvoice;
{
	switch(mode)
	   {
	    case MIPL_TEXTANDVOICE:
	    case MIPL_TEXTONLY:
	    case MIPL_TEXTPREFERRED:
		return(1);
	    case MIPL_VOICEPREFERRED:
		if (!gotvoice) return(1);
		break;
	   }
	return(0);
}

int determine_voice(mode,gottext)
int mode,gottext;
{
	switch(mode)
	   {
	    case MIPL_TEXTANDVOICE:
	    case MIPL_VOICEONLY:
	    case MIPL_VOICEPREFERRED:
		return(1);
	    case MIPL_TEXTPREFERRED:
		if (!gottext) return(1);
		break;
	   }
	return(0);
}

int gotv(tag,i)
TAG *tag;
int i;
{
int	j;

	for(j = 0; j < tag->tags[i]->npairs; j++)
	   {
		if (!tag->tags[i]->names[j]) continue;
		if (!strcmp(tag->tags[i]->names[j],"VOICE"))
			if (tag->tags[i]->values[j]) return(1);
	   }
	return(0);
}

void killtemp()
{
#ifndef	NORECORD
	unlink(rectemp);
#endif
	return;
}

int finddelim(str,strp,delimchr,quotechr)
char *str;
char *strp[];
{
int	i,inquo;

	inquo = 0;
	i = 0;
	strp[i++] = str;
	if (!*str) 
	   {
		strp[0] = 0;
		return(0);
	   }
	for(; *str; str++)
	   {
		if (*str == quotechr)
		   {
			if (inquo)
			   {
				*str = 0;
				inquo = 0;
			   }
			else
			   {
				strp[i - 1] = str + 1;
				inquo = 1;
			   }
		   }
		if ((*str == delimchr) && (!inquo))
		   {
			*str = 0;
			strp[i++] = str + 1;
		   }
	   }
	strp[i] = 0;
	return(i);

}

gyrate(curtag)
TAG *curtag;
{
char	amp[MAXAMPR];
TAG	*newtag,*newtag1;
char	*textp,*namep,*valuep,*firstname;
int	c,i,j,k,tavail;

	for(;;)
	   {
		c = mip_cin();
		if (c == EOF) return(-1);
		/* okay, so its in text state */
		textp = malloc(BIGONE + 1);
		if (!textp)
		   {
			perror("malloc -- textp");
			killtemp();
			exit(255);
		   }
		tavail = BIGONE;
		i = k = 0;
		for(;;)
		   {
			if (c == '<')
			   {
				textp[i] = 0;
				break;
			   }
			if (c == '&') /* if in an ampersand thingy */
			   {
				c = 0;
				for(amp[j = 0] = 0;j < MAXAMPR;j++)
				   {
					c = mip_cin();
					if (c == EOF) return(-1);
					if (c == ';')
					   {
						amp[j] = 0;
						if (!strcmp(amp,"amp"))
						   {
							c = '&';
							break;
						   }
						if (!strcmp(amp,"lt"))
						   {
							c = '<';
							break;
						   }
						if (!strcmp(amp,"gt"))
						   {
							c = '>';
							break;
						   }
						c = 0;
						break;
					   }
					amp[j] = c;
				   }				
			   }				
			if (c)
			   {
				if (i == tavail)
				   {
					textp = realloc(textp,tavail + BIGONE + 1);
					if (!textp)
					   {
						perror("realloc to grow textp");
						killtemp();
						exit(255);
					   }
					tavail += BIGONE;
				   }
				if ((k) || (c >= ' ')) textp[i++] = c;
				k++;
				textp[i] = 0;
			   }
			c = mip_cin();
			if (c == EOF) return(-1);
		   }
		  /* get rida trailing LF */
		if (i && (textp[i - 1] < ' '))
		   {
			textp[--i] = 0;
		   }
		if (i)
		   {
			textp = realloc(textp,i + 1);  /* shrink to reasonable size */
			if (!textp)
			   {
				perror("textp realloc for shrinkage");
				killtemp();
				exit(255);
			   }
			if (curtag->ntags < MAXTAGS)
			   {
				newtag1 = (TAG *) malloc(sizeof(TAG));
				if (!newtag1)
				   {
					perror("malloc newtag");
					killtemp();
					exit(255);
				   }
				memset(newtag1,0,sizeof(TAG)); /* set all to zero */
				curtag->tags[curtag->ntags++] = newtag1;
				newtag1->names[0] = 0;
				newtag1->values[0] = textp;
				newtag1->npairs = 1;
			   }
			else
			   {
				if (textp) free(textp);
			   }
		   }
		else
		   {
			free(textp); /* throw away if nothing in there */
		   }
		/* so we are now in a tag, so lets make a new one first */
		newtag = (TAG *) malloc(sizeof(TAG));
		if (!newtag)
		   {
			perror("malloc newtag");
			killtemp();
			exit(255);
		   }
		memset(newtag,0,sizeof(TAG)); /* set all to zero */
		namep = valuep = firstname = 0;
		for(i = 0,j = 0;;)
		   {
			c = mip_cin();
			if (c == EOF) return(-1);
			if (c == '>')
			   {
				if (i == 1) namep[k] = 0;
				if (i == 3) valuep[k] = 0;
				if (namep)
				   {
					if (newtag->npairs < MAXPARAMS)
					   {
						newtag->names[newtag->npairs] = namep;
						newtag->values[newtag->npairs++] = valuep;
					   }
					else
					   {
						if (namep) free(namep);
						if (valuep) free(valuep);
					   }
				   }
				  break; /* if end of tag */
			   }
			switch(i) /* present state */
			   {
			    case 0: /* before name */
				  /* keep going past white space */
				if (isspace(c)) continue;
				if (c == '\n') continue;
				if (c == '=') /* this is a value */
				   {
					valuep = malloc(SMALLONE + 1);
					if (!valuep)
					   {
						perror("valuep malloc");
						killtemp();
						exit(255);
					   }
 					i = 2; /* set to value state */
					j = 0; /* reset quoting flag */
					k = 0; /* clear counter */
					continue;
				   }
				if (namep)
				   {
					if (newtag->npairs < MAXPARAMS)
					   {
						newtag->names[newtag->npairs] = namep;
						newtag->values[newtag->npairs++] = valuep;
					   }
					else
					   {
						if (namep) free(namep);
						if (valuep) free(valuep);
					   }
					namep = valuep = 0;
				   }
				  /* alloc the name pointer */
				namep = malloc(SMALLONE + 1);
				if (!namep)
				   {
					perror("namep malloc");
					killtemp();
					exit(255);
				   }
				i = 1; /* set to name state */
				j = 0; /* reset quoting flag */
				k = 0; /* clear counter */
				if (c == '\"') j = 1;
				else namep[k++] = toupper(c);
				continue;
			    case 1: /* "in name" state */
				if (!firstname) firstname = namep;
				if (c == '\"') /* if a quote */
				   {
					if (!j) /* if not in quote */
					   {
						j = 1; /* we are now */
						continue;
					   }
					else
					   {
						j = 0;
						namep[k] = 0;
						i = 0; /* next lexan state */
						continue;
					   }
				   }
				  /* if space, and not in quote */
				if ((c == ' ') && (!j))
				   {
					namep[k] = 0;
					if (chkcomment(namep))
					   {
						free(namep);
						free(newtag);
						return(1);  /* indicate comment, restart */
					   }
					i = 0; /* before name state */
					continue;
				   }
				if ((c == '=') && (!j))
				   {
					namep[k] = 0;
					if (chkcomment(namep))
					   {
						free(namep);
						free(newtag);
						return(1);  /* indicate comment, restart */
					   }
					i = 2; /* before value state */
					continue;
				   }
				if (k < SMALLONE) namep[k++] = toupper(c);
				continue;
			    case 2: /* before value */
				  /* keep going past white space */
				if (isspace(c)) continue;
				if (c == '\n') continue;
				valuep = malloc(SMALLONE + 1);
				if (!valuep)
				   {
					perror("valuep malloc");
					killtemp();
					exit(255);
				   }
				i = 3; /* set to value state */
				j = 0; /* reset quoting flag */
				k = 0; /* clear counter */
				if (c == '\"') j = 1;
				else valuep[k++] = c;
				continue;
			   case 3: /* "in value" state */
				if (c == '\"') /* if a quote */
				   {
					if (!j) /* if not in quote */
					   {
						j = 1; /* we are now */
						continue;
					   }
					else
					   {
						j = 0;
						valuep[k] = 0;
						i = 0; /* next lexan state */
						continue;
					   }
				   }
				  /* if space, and not in quote */
				if ((c == ' ') && (!j))
				   {
					valuep[k] = 0;
					i = 0; /* next lexan state */
					continue;
				   }
				if (k < SMALLONE) valuep[k++] = c;
				continue;
			   }
		   }						
		/* if end of this tag, exit loop */
		if (firstname && (*firstname == '/')) 
		   {
			freetag(newtag);
			break;
		   }
		if (curtag->ntags < MAXTAGS)
		   {
			  /* save value in curtag struct */
			curtag->tags[curtag->ntags++] = newtag;
		   }
		if (firstname) /* if standalone tag, deal with it */
		   {
			if (!strcmp(firstname,"NAME")) continue;
			if (!strcmp(firstname,"EXIT")) continue;
			if (!strcmp(firstname,"GOTO")) continue;
			if (!strcmp(firstname,"HIDDEN")) continue;
			if (!strcmp(firstname,"VOICE")) continue;
			if (!strcmp(firstname,"VOICERATE")) continue;
			if (!strcmp(firstname,"PREFER")) continue;
			if (!strcmp(firstname,"NOINTERRUPT")) continue;
			if (!strcmp(firstname,"ONINTERRUPT")) continue;
		   }
		do i = gyrate(newtag); /* recursively call itself */
		while (i == 1); /* again if comment */
		  /* if was over n stuff */
		if ((i == -1) || (curtag->ntags >= MAXTAGS))
		   {
			freetag(newtag);
		   }
	   }
	return(0);
}

chkcomment(str)
char *str;
{
int	c,i;

	if (strcmp(str,"!--")) return(0);
	for(i = 0;;)
	   {
		c = mip_cin();
		if (c == EOF) 
		   {
			killtemp();
			exit(0);
		   }
		switch(i)
		   {
		    case 0:
		    case 1:
			if (c != '-') i = 0; else i++;
			break;
		    case 2:
			if (c != '>') i = 0; else i++;
			break;
		   }
		if (i == 3) break;
	   }
	return(1);
}


int dotextnvoice(tagp,chained,vflag,label)
TAG *tagp;
int chained;
int vflag;
char *label;
{
int	f,i,j,k,intf,gotvoice = 0,gottext = 0,got_int = 0,pres_mode;
char	buf[10],*inturl;

	vok = 0;
	gotvoice = vflag;
	pres_mode = out_mode;
	if (!chained) mipl_flush(MIPL_READ);
	intf = 1;
	if (label && (!*label)) label = 0;
	f = 1;
	if (label) f = 0;
	/* get interrupt status n other stuff */
	for(i = 0; i < tagp->ntags; i++)
	    {
		if (label && (!f))
		   {
			if (!tagp->tags[i]->names[0]) continue;
			if (!tagp->tags[i]->values[0]) continue;
			if (strcmp(tagp->tags[i]->names[0],"NAME")) continue;
			if (strcmp(tagp->tags[i]->values[0],label)) continue;
			f = 1;
		   }
		if (!tagp->tags[i]->names[0])
		   {
			gottext = 1;
			continue;
		   }
		if (!strcmp(tagp->tags[i]->names[0],"NOINTERRUPT"))
		   {
			intf = 0;
			continue;
		   }
		if (!strcmp(tagp->tags[i]->names[0],"VOICE"))
		   {
			gotvoice = 1;
			continue;
		   }
		if (!strcmp(tagp->tags[i]->names[0],"NAME")) continue;
		if (!strcmp(tagp->tags[i]->names[0],"VOICERATE")) continue;
		if (!strcmp(tagp->tags[i]->names[0],"PREFER")) continue;
		if (!strcmp(tagp->tags[i]->names[0],"ONINTERRUPT")) continue;
		break;
	   }
	f = 1;
	if (label) f = 0;
	if (!chained) miplstr_init();
	for(i = 0; i < tagp->ntags; i++)
	   {
		if (label && (!f))
		   {
			if (!tagp->tags[i]->names[0]) continue;
			if (!tagp->tags[i]->values[0]) continue;
			if (strcmp(tagp->tags[i]->names[0],"NAME")) continue;
			if (strcmp(tagp->tags[i]->values[0],label)) continue;
			f = 1;
		   }
		if (!tagp->tags[i]->names[0])
		   {
			if (determine_text(pres_mode,gotvoice))
			   {
				if (tagp->tags[i]->values[0]) 
					mipltext_add(tagp->tags[i]->values[0]);
			   }
			continue;
		   }
		if (!strcmp(tagp->tags[i]->names[0],"VOICE"))
		   {
			if (determine_voice(pres_mode,gottext))
			   {
				for(j = 0; j < tagp->tags[i]->npairs; j++) 
				   {
					if (!tagp->tags[i]->names[j]) continue;
					if (!tagp->tags[i]->values[j]) continue;
					if (!strcmp(tagp->tags[i]->names[j],"TEXT") && determine_text(out_mode,1))
						mipltext_add(tagp->tags[i]->values[j]);
					else miplvstr_cmd(tagp->tags[i]->names[j],
						tagp->tags[i]->values[j]);
					if ((!chained) && mipl_out(intf))
					   {
						i = tagp->ntags;
						got_int = 1;
						break;
					   }   
				   }
			   }
			continue;
		   }
		if (!strcmp(tagp->tags[i]->names[0],"NOINTERRUPT")) continue;
		if (!strcmp(tagp->tags[i]->names[0],"ONINTERRUPT")) continue;
		if (!strcmp(tagp->tags[i]->names[0],"NAME")) continue;
		if (!strcmp(tagp->tags[i]->names[0],"VOICERATE"))
		   {
			dovoicerate(tagp->tags[i]);
			continue;
		   }
		if (!strcmp(tagp->tags[i]->names[0],"PREFER"))
		   {
			if (!tagp->tags[i]->values[0])
			   {
				pres_mode = out_mode;
				continue;
			   }
			if (!strcmp(tagp->tags[i]->values[0],"BOTH"))
			   {
				if (determine_text(out_mode,0) && determine_voice(out_mode,0))
				   {
					pres_mode = MIPL_TEXTANDVOICE;
				   }
				continue;
			   }
			if (!strcmp(tagp->tags[i]->values[0],"TEXT"))
			   {
				if (determine_text(out_mode,0)) 
					pres_mode = MIPL_TEXTPREFERRED;
				continue;
			   }
			if (!strcmp(tagp->tags[i]->values[0],"VOICE"))
			   {
				if (determine_voice(out_mode,0)) 
					pres_mode = MIPL_VOICEPREFERRED;
				continue;
			   }
			if (!strcmp(tagp->tags[i]->values[0],"NORMAL"))
			   {
				pres_mode = out_mode;
				continue;
			   }
			continue;
		   }
		if (f) break;
	   }
	vok = determine_voice(out_mode,gottext);
	if (!chained)
	   {
		if (mipl_out(intf)) got_int = 1;
	   }
	  /* if didnt get interrupt, return here */
	if (!got_int) return(intf);
	  /* get touchtone that was hit, and return if error */
	if (!mipl_prompt(buf,1,1,0,0)) return(intf);
	f = 1;
	if (label) f = 0;
	for(i = 0; i < tagp->ntags; i++)
	   {
		if (label && (!f))
		   {
			if (!tagp->tags[i]->names[0]) continue;
			if (!tagp->tags[i]->values[0]) continue;
			if (strcmp(tagp->tags[i]->names[0],"NAME")) continue;
			if (strcmp(tagp->tags[i]->values[0],label)) continue;
			f = 1;
		   }
		if (!tagp->tags[i]->names[0]) continue;
		  /* ignore that which is not ONINTERRUPT */
		if (strcmp(tagp->tags[i]->names[0],"ONINTERRUPT")) continue;
		inturl = 0;
		k = 0;
		for(j = 0; j < tagp->tags[i]->npairs; j++) 
		   {
			if (!tagp->tags[i]->names[j]) continue;
			if (!tagp->tags[i]->values[j]) continue;
			if (!strcmp(tagp->tags[i]->names[j],"KEY"))
			   {
				  /* if key specified and was not found, set flag */
				if (!strchr(tagp->tags[i]->values[j],buf[0])) k = 1;
			   }
			if (!strcmp(tagp->tags[i]->names[j],"HREF")) 
			   {
				inturl = tagp->tags[i]->values[j];
			   }
		   }
		if (k || (!inturl)) continue;
		if (openurl(inturl,0))
		   {
			sprintf(outbuf,"cannot open/load URL %s",inturl);
			mipl_error(outbuf);
			return(intf);
		   }
		return(2);
	   }				
	return(intf);
}

int runscript(label)
char *label;
{
int	i,j,k;

#ifdef	D40V2
	mipl_voicerate(6000);
#else
	mipl_voicerate(8000);
#endif
	mipl_flush(MIPL_READ);
	for(i = 0; i < maintag.ntags; i++)
	   {
		if (!maintag.tags[i]->names[0]) continue; /* ignore null ones */
		if (!maintag.tags[i]->values[0]) continue; /* ignore null ones */
		if (strcmp(maintag.tags[i]->names[0],"VOICERATE")) continue;
		dovoicerate(maintag.tags[i]);
	   }
	 /* output text and voice */
	if (dotextnvoice(&maintag,0,0,label) == 2) return(1);
	j = 1;
	if (label[0]) j = 0;
 	/* go through and execute the execution items */
	for(i = 0; i < maintag.ntags; i++)
	   {
		if (!maintag.tags[i]->names[0]) continue; /* ignore null ones */
		  /* if label hasnt been reached yet, and this is a label */
		if (!j)
		   {
			if (!strcmp(maintag.tags[i]->names[0],"NAME"))
			   {
				  /* if this is the correct label */
				if (!strcmp(maintag.tags[i]->values[0],label))
				   {
					j = 1; /* set found flag */
				   }
			   }
			else
			   {
				continue;
			   }
		   }
		  /* if to exit, do so */
		if (!strcmp(maintag.tags[i]->names[0],"EXIT")) break;
		if (!strcmp(maintag.tags[i]->names[0],"MENU"))
			if (domenu_select(maintag.tags[i],0)) return(1);
		if (!strcmp(maintag.tags[i]->names[0],"GOTO"))
			if (dogoto(maintag.tags[i])) return(1);
		if (!strcmp(maintag.tags[i]->names[0],"FORM"))
			if (doform(maintag.tags[i])) return(1);
#ifndef	NORECORD
                if (!strcmp(maintag.tags[i]->names[0],"RECORD"))
                        if (dorecord(maintag.tags[i])) return(1);     
#endif
	   }
	  /* if label not found */
	if ((!j) && (i >= maintag.ntags))
	   {
		sprintf(outbuf,"label not found: %s",label);
		mipl_error(outbuf);
	   }		
	killtemp();
	return(0);
}
	
dumptag(tag) /* dump the contents of a tag structure */
TAG *tag;
{
register int	i;

	for(i = 0; i < tag->npairs; i++)
	   {
		printf("%s - %s\n",tag->names[i],tag->values[i]);
	   }
	printf("\n");
	for(i = 0; i < tag->ntags; i++) 
	   {
		printf("%s->",tag->names[0]);
		dumptag(tag->tags[i]);
	   }
	return;
}

dumpform() /* dump the contents of a tag structure */
{
register int	i;

	printf("\nForm contents:\n\n");
	for(i = 0; i < form.npairs; i++)
	   {
		printf("%s - %s\n",form.names[i],form.values[i]);
	   }
	printf("\n");
	return;
}

freetag(tag) /* free all the resources associated with a tag structure */
TAG *tag;
{
register int	i;

	for(i = 0; i < tag->ntags; i++) freetag(tag->tags[i]);
	for(i = 0; i < tag->npairs; i++)
	   {
		if (tag->names[i]) free(tag->names[i]);
		if (tag->values[i]) free(tag->values[i]);
	   }
	if (tag == &maintag) memset(tag,0,sizeof(TAG)); else free(tag);
	return;
}

freeform() /* free all the resources associated with a the form */
{
register int	i;

	for(i = 0; i < form.npairs; i++)
	   {
		free(form.names[i]);
		free(form.values[i]);
	   }
	memset(&form,0,sizeof(FORM));
	return;
}

static char *made_selection(tag,tag1,selectflag)
TAG *tag,*tag1;
int selectflag;
{
int	i,j,k;
char	*cp;

	for(j = 0,k = 0; j < tag1->npairs; j++)
	   {
		if (!tag1->names[j]) continue;
		if (!selectflag) /* if was a menu */
		   {
			if (!strcmp(tag1->names[j],"HREF"))
				return(tag1->values[j]);
			continue;
		   }
		/* musta been a select then */
		if (!strcmp(tag1->names[j],"VALUE"))
		   {
			if (tag1->values[j])
			   {
				cp = malloc(mimelen(tag1->values[j]) + 1);
				if (!cp)
				   {
					perror("malloc");
					killtemp();
					exit(255);
				   }
				form.values[form.npairs] = cp;
				mimecpy(form.values[form.npairs],tag1->values[j]);
				k = 1;
			   }
		   }
	   }
	if (k)
	   {
		for(i = 0; i < tag->npairs; i++)
		   {
			if (!tag->names[i]) continue; /* skip over null name */
			if (!strcmp(tag->names[i],"NAME"))
			   {
				if (tag->values[i])
				   {
					cp = malloc(mimelen(tag->values[i]) + 1);
					if (!cp)
					   {
						perror("malloc");
						killtemp();
						exit(255);
					   }
					form.names[form.npairs] = cp;
					mimecpy(form.names[form.npairs],tag->values[i]);
					break;
				   }
			   }
		   }
		form.npairs++;
		return("");
	   }
	return(NULL);
}

int domenu_select(tag,selectflag)  /* execute the menu stuff */
TAG *tag;
int selectflag;
{
int	i,j,k,m,n,try,tryi,trymax,intf,nobs;
char	a[300],*cp,*url,*tryurl;

	nobs = 0;
	tryi = 0;
	trymax = 1;
	tryurl = 0;
	  /* look for tries spec */
	for(i = 0; i < tag->npairs; i++)
	   {
		if (!tag->names[i]) continue;
		if (strcmp(tag->names[i],"TRIES")) continue;
		if (!tag->values[i]) continue;
		cp = strchr(tag->values[i],',');
		if (!cp)
		   {
			tryurl = tag->values[i];
		   }
		else
		   {
			*cp = 0;
			trymax = atoi(tag->values[i]);
			tryurl = cp + 1;
			tryi = 1;
		   }
		break;
	   }

	for(try = 0;try < trymax;try += tryi)
	   {
		mipl_flush(MIPL_READ);
		miplstr_init();
		intf = dotextnvoice(tag,1,0,NULL);
		if (intf == 2) return(1);
		m = 1;
		  /* now for all the selections */
		for(i = 0; i < tag->ntags; i++)
		   {
			if (!tag->tags[i]->names[0]) continue;
			if (strcmp(tag->tags[i]->names[0],"SELECTION")) continue;
                        if (!tag->tags[i]->values[0]) continue;
			if (dotextnvoice(tag->tags[i],1,gotv(tag,i),NULL) == 2) return(1);
			if (vok) for(j = 0; j < tag->tags[i]->npairs; j++)
			   {
				if (!tag->tags[i]->names[j]) continue;
				if (!strcmp(tag->tags[i]->names[j],"VOICE"))
					miplvstr_url(tag->tags[i]->values[j]);
			   }
                        k = strlen(tag->tags[i]->values[0]);
                        if (k > m) m = k; /* get longest selection value */ 
		   }
		  /* now for menuend */
		for(i = 0; i < tag->ntags; i++)
		   {
			if (!tag->tags[i]->names[0]) continue;
			if (strcmp(tag->tags[i]->names[0],"MENUEND")) continue;
			if (dotextnvoice(tag->tags[i],1,gotv(tag,i),NULL) == 2) return(1);
			for(j = 0; j < tag->tags[i]->npairs; j++)
			   {
				if (!tag->tags[i]->names[j]) continue;
				if (!tag->tags[i]->values[j]) continue;
				if (!strcmp(tag->tags[i]->names[j],"VOICE"))
					if (vok) miplvstr_url(tag->tags[i]->values[j]);
			   }
		   }
		k = 0; /* timeout time */
		url = 0;
		  /* now for timeout */
		for(i = 0; i < tag->ntags; i++)
		   {
			if (!tag->tags[i]->names[0]) continue;
			if (strcmp(tag->tags[i]->names[0],"TIMEOUT")) continue;
			for(j = 0; j < tag->tags[i]->npairs; j++)
			   {
				if (!tag->tags[i]->names[j]) continue;
				if (!strcmp(tag->tags[i]->names[j],"TIME"))
				   {
					if (tag->tags[i]->values[j])
						sscanf(tag->tags[i]->values[j],"%d",&k);
				   }
			   }
		   }
		if (m > (sizeof(a) - 1)) m = sizeof(a) - 1;
		url = 0;
		if (!mipl_prompt(a,m,intf,k,0))
		   {
			a[0] = 0;
			  /* now for timeout */
			for(i = 0; i < tag->ntags; i++)
			   {
				if (!tag->tags[i]->names[0]) continue;
				if (strcmp(tag->tags[i]->names[0],"TIMEOUT")) continue;
				if (dotextnvoice(tag->tags[i],0,gotv(tag,i),NULL) == 2) return(1);
				miplstr_init();
				for(j = 0; j < tag->tags[i]->npairs; j++)
				   {
					if (!tag->tags[i]->names[j]) continue;
					if (!strcmp(tag->tags[i]->names[j],"HREF"))
						url = tag->tags[i]->values[j];
					if (!strcmp(tag->tags[i]->names[j],"VOICE"))
						if (tag->tags[i]->values[j]) 
							if (vok) miplvstr_url(tag->tags[i]->values[j]);
				   }
				mipl_out(1);
			   }
			if (url)
			   {
				if (openurl(url,0))
				   {
					sprintf(outbuf,"cannot open/load URL %s",url);
					mipl_error(outbuf);
					continue;
				   }
				return(1);
			   }
			continue;
		   }
		/* was normal input */
		if (selectflag && (strchr(a,'\033') || (!a[0])))  /* if escape */
		   {
			  /* look for escape spec */
			for(i = 0; i < tag->npairs; i++)
			   {
				if (!tag->names[i]) continue;
				if (strcmp(tag->names[i],"ESCAPE")) continue;
				url = tag->values[i];
			   }
		   }
		if ((!selectflag) && (strchr(a,'\033'))) a[0] = 0;
		  /* look for matching selection */
		if (!url) for(i = 0; i < tag->ntags; i++)
		   {
			if (!tag->tags[i]->names[0]) continue;
			if (strcmp(tag->tags[i]->names[0],"SELECTION")) continue;
			if (!tag->tags[i]->values[0]) continue;
			if (strcmp(tag->tags[i]->values[0],a)) continue;
			url = made_selection(tag,tag->tags[i],selectflag);
			  /* if selected and loaded, go back */
			if (selectflag && (!strcmp(url,""))) return(0);
		   }
		if (!url)  /* invalid response */
		   {
			  /* find the invalid if any */
			for(i = 0; i < tag->ntags; i++)
			   {
				if (!tag->tags[i]->names[0]) continue;
				if (strcmp(tag->tags[i]->names[0],"INVALID")) continue;
				if (dotextnvoice(tag->tags[i],0,gotv(tag,i),NULL) == 2) return(1);
				miplstr_init();
				for(j = 0; j < tag->tags[i]->npairs; j++)
				   {
					if (!tag->tags[i]->names[j]) continue;
					if (!strcmp(tag->tags[i]->names[j],"HREF"))
						url = tag->tags[i]->values[j];
					if (!strcmp(tag->tags[i]->names[j],"VOICE"))
						if (tag->tags[i]->values[j]) 
							if(vok) miplvstr_url(tag->tags[i]->values[j]);
				   }
				mipl_out(1);
			   }
		   }
		if (!url) continue; /* still invalid */
		if (openurl(url,0))
		   {
			sprintf(outbuf,"cannot open/load URL %s",url);
			mipl_error(outbuf);
			continue;
		   }
		return(1); /* return to execute new url */
	   }
	if (!tryurl)
	   {
		mipl_error("maxtries exceeded, but URL not specified");
		killtemp();
		exit(0);
	   }
	if (openurl(tryurl,0))
	   {
		sprintf(outbuf,"cannot open/load URL %s",tryurl);
		mipl_error(outbuf);
		killtemp();
		return(0);
	   }
	return(1); /* return to execute new url */
}

int dogoto(tag)  /* go to a URL */
TAG *tag;
{
int	i,j,k;
char	*url = 0;
extern	int firsttdd;

	for(j = 0; j < tag->npairs; j++)
	   {
		if (!tag->names[j]) continue;
		if (!tag->values[j]) continue;
		if (!strcmp(tag->names[j],"TDD"))
		   {
			out_mode = MIPL_TEXTONLY;
			if (!firsttdd)
			   {
				firsttdd = 1;
				usleep(2000000);  /* wait 2 secs */
				if (zap_disatone(zap) < 0) exit(0);
				usleep(1000000);
			   }				
		   }
		if (!strcmp(tag->names[j],"NOTDD"))
		   {
			out_mode = MIPL_VOICEONLY;
		   }
		if (strcmp(tag->names[j],"HREF")) continue;
		if (tag->values[j]) url = tag->values[j];
	   }
	if (!url)
	   {
		mipl_error("NO URL specicified in GOTO statement");
		killtemp();
		exit(0);
	   }
	if (openurl(url,0))
	   {
		sprintf(outbuf,"cannot open/load URL %s",url);
		mipl_error(outbuf);
		killtemp();
		exit(0);
	   }
	return(1);
}

int doform(tag)
TAG *tag;
{
int	i,j,k;
char	*url;

	freeform();  /* clear it out */
 	/* go through and execute the execution items */
	for(i = 0; i < tag->ntags; i++)
	   {
		if (!tag->tags[i]->names[0]) continue; /* ignore null ones */
		if (!strcmp(tag->tags[i]->names[0],"HIDDEN"))
			dohidden(tag->tags[i]);
		if (!strcmp(tag->tags[i]->names[0],"SUBMIT"))
			dosubmit(tag->tags[i]);
		if (!strcmp(tag->tags[i]->names[0],"SELECT"))
			if (domenu_select(tag->tags[i],1)) return(1);
		if (!strcmp(tag->tags[i]->names[0],"DATAIN"))
			if (dodatain(tag->tags[i],1)) return(1);
		if (!strcmp(tag->tags[i]->names[0],"ASK"))
			if (doask(tag->tags[i],1)) return(1);
	   }
	dumpform();
	k = GETMETHOD;
	for(j = 0; j < tag->npairs; j++)
	   {
		if (!tag->names[j]) continue;
		if (!strcmp(tag->names[j],"ACTION"))
		   {
			if (tag->values[j]) url = tag->values[j];
		   }
		if (!strcmp(tag->names[j],"METHOD"))
		   {
			if (tag->values[j])
			   {
				if (!strcmp(tag->values[j],"POST")) k = POSTFORM;
			   }
		   }
	   }
	if (!url)
	   {
		mipl_error("NO ACTION URL specicified in form");
		return(0);
	   }
	if (openurl(url,k))
	   {
		sprintf(outbuf,"cannot open/load URL %s",url);
		mipl_error(outbuf);
		return(0);
	   }
	return(1);
}

dohidden(tag)
TAG *tag;
{

int	i,j,k;
char	*cp;

	for(i = 0; i < tag->npairs; i++)
	   {
		j = 0;
		if (!tag->names[i]) continue; /* skip over null name */
		if (!strcmp(tag->names[i],"NAME"))
		   {
			if (tag->values[i])
			   {
				cp = malloc(mimelen(tag->values[i]) + 1);
				if (!cp)
				   {
					perror("malloc");
					killtemp();
					exit(255);
				   }
				form.names[form.npairs] = cp;
				mimecpy(form.names[form.npairs],tag->values[i]);
				j = 1;
			   }
		   }
		if (!strcmp(tag->names[i],"VALUE"))
		   {
			if (tag->values[i])
			   {
				cp = malloc(mimelen(tag->values[i]) + 1);
				if (!cp)
				   {
					perror("malloc");
					killtemp();
					exit(255);
				   }
				form.values[form.npairs] = cp;
				mimecpy(form.values[form.npairs],tag->values[i]);
				j = 1;
			   }
		   }
	   }
	  /* bump index if appropriate */
	form.npairs += j;
	return(0);
}


dosubmit(tag)
TAG *tag;
{

int	i,j,k;
char	*cp;

	for(i = 0,j = 0; j < tag->npairs; j++)
	   {
		if (!tag->names[j]) continue;
		if (!strcmp(tag->names[j],"VOICE"))
			if (tag->values[j])  i = 1;
	   }
	k = dotextnvoice(tag,0,i,NULL);
	if (k == 2) return(1);
	miplstr_init();
	for(j = 0; j < tag->npairs; j++)
	   {
		if (!tag->names[j]) continue;
		if (!strcmp(tag->names[j],"VOICE"))
			if (tag->values[j]) 
				if (vok) miplvstr_url(tag->values[j]);
	   }
	mipl_out(k);
	dohidden(tag);
	return(0);
}

int dodatain(tag)  /* get data from user */
TAG *tag;
{
int	i,j,k,m,try,tryi,trymax,intf;
char	a[300],*cp,*url,*tryurl;

	tryi = 0;
	trymax = 1;
	tryurl = 0;
	  /* look for tries spec */
	for(i = 0; i < tag->npairs; i++)
	   {
		if (!tag->names[i]) continue;
		if (strcmp(tag->names[i],"TRIES")) continue;
		if (!tag->values[i]) continue;
		cp = strchr(tag->values[i],',');
		if (!cp)
		   {
			tryurl = tag->values[i];
		   }
		else
		   {
			*cp = 0;
			trymax = atoi(tag->values[i]);
			tryurl = cp + 1;
			tryi = 1;
		   }
		break;
	   }
	miplstr_init();
	for(try = 0;try < trymax;try += tryi)
	   {
		mipl_flush(MIPL_READ);
		intf = dotextnvoice(tag,1,0,NULL);
		if (intf == 2) return(1);
		k = 0; /* timeout time */
		m = 0; /* max */
		url = 0;
		  /* get the timeout time */
		for(i = 0; i < tag->ntags; i++)
		   {
			if (!tag->tags[i]->names[0]) continue;
			if (strcmp(tag->tags[i]->names[0],"TIMEOUT")) continue;
			for(j = 0; j < tag->tags[i]->npairs; j++)
			   {
				if (!tag->tags[i]->names[j]) continue;
				if (!strcmp(tag->tags[i]->names[j],"TIME"))
				   {
					if (tag->tags[i]->values[j])
						sscanf(tag->tags[i]->values[j],"%d",&k);
				   }
			   }
		   }
		  /* get the MAX parameter */
		for(i = 0; i < tag->npairs; i++)
		   {
			if (!tag->names[i]) continue;
			if (strcmp(tag->names[i],"MAX")) continue;
			if (!tag->values[i]) break;
			sscanf(tag->values[i],"%d",&m);
		   }
		if (m > (sizeof(a) - 1)) m = sizeof(a) - 1;
		if (!mipl_prompt(a,m,intf,k,MIPL_DATAIN))
		   {
			a[0] = 0;
			  /* now for timeout */
			for(i = 0; i < tag->ntags; i++)
			   {
				if (!tag->tags[i]->names[0]) continue;
				if (strcmp(tag->tags[i]->names[0],"TIMEOUT")) continue;
				if (dotextnvoice(tag->tags[i],0,gotv(tag,i),NULL) == 2) return(1);
				miplstr_init();
				for(j = 0; j < tag->tags[i]->npairs; j++)
				   {
					if (!tag->tags[i]->names[j]) continue;
					if (!strcmp(tag->tags[i]->names[j],"HREF"))
						url = tag->tags[i]->values[j];
					if (!strcmp(tag->tags[i]->names[j],"VOICE"))
						if (tag->tags[i]->values[j]) 
							if (vok) miplvstr_url(tag->tags[i]->values[j]);
				   }
				mipl_out(1);
			   }
			if (url)
			   {
				if (openurl(url,0))
				   {
					printf(outbuf,"cannot open/load URL %s",url);
					mipl_error(outbuf);
					continue;
				   }
				return(1);
			   }
			continue;
		   }
		/* it was normal happy input */
		if (strchr(a,'\033'))  /* if escape */
		   {
			  /* look for escape spec */
			for(i = 0; i < tag->npairs; i++)
			   {
				if (!tag->names[i]) continue;
				if (strcmp(tag->names[i],"ESCAPE")) continue;
				url = tag->values[i];
			   }
			if (!url) continue; /* still invalid */
			if (openurl(url,0))
			   {
				sprintf(outbuf,"cannot open/load URL %s\n",url);
				mipl_error(outbuf);
				continue;
			   }
			return(1); /* return to execute new url */
		   }
		if (!a[0])  /* invalid response */
		   {
			  /* find the invalid if any */
			for(i = 0; i < tag->ntags; i++)
			   {
				if (!tag->tags[i]->names[0]) continue;
				if (strcmp(tag->tags[i]->names[0],"INVALID")) continue;
				if (dotextnvoice(tag->tags[i],0,gotv(tag,i),NULL) == 2) return(1);
				miplstr_init();
				for(j = 0; j < tag->tags[i]->npairs; j++)
				   {
					if (!tag->tags[i]->names[j]) continue;
					if (!strcmp(tag->tags[i]->names[j],"HREF"))
						url = tag->tags[i]->values[j];
					if (!strcmp(tag->tags[i]->names[j],"VOICE"))
						if (tag->tags[i]->values[j]) 
							if(vok) miplvstr_url(tag->tags[i]->values[j]);
				   }
				mipl_out(1);
			   }
			if (!url) continue; /* still invalid */
			if (openurl(url,0))
			   {
				sprintf(outbuf,"cannot open/load URL %s\n",url);
				mipl_error(outbuf);
				continue;
			   }
			return(1); /* return to execute new url */
		   }
		  /* crop to MAX length, if any */
		if ((m) && (strlen(a) > m)) a[m] = 0;
		cp = malloc(mimelen(a) + 1);
		if (!cp)
		   {
			perror("malloc");
			killtemp();
			exit(255);
		   }
		form.values[form.npairs] = cp;
		mimecpy(form.values[form.npairs],a);
		for(i = 0; i < tag->npairs; i++)
		   {
			if (!tag->names[i]) continue; /* skip over null name */
			if (!strcmp(tag->names[i],"NAME"))
			   {
				if (tag->values[i])
				   {
					cp = malloc(mimelen(tag->values[i]) + 1);
					if (!cp)
					   {
						perror("malloc");
						killtemp();
						exit(255);
					   }
					form.names[form.npairs] = cp;
					mimecpy(form.names[form.npairs],tag->values[i]);
					break;
				   }
			   }
		   }
		form.npairs++;
		return(0);
	   }
	if (!tryurl)
	   {
		mipl_error("maxtries exceeded, but URL not specified");
		killtemp();
		exit(0);
	   }
	if (openurl(tryurl,0))
	   {
		sprintf(outbuf,"cannot open/load URL %s\n",tryurl);
		mipl_error(outbuf);
		killtemp();
		return(0);
	   }
	return(1); /* return to execute new url */
}


int doask(tag)
TAG *tag;
{
int	i,j,k,try,tryi,trymax,intf;
char	a[80],*cp,*url,*tryurl;

	tryi = 0;
	trymax = 1;
	tryurl = 0;
	  /* look for tries spec */
	for(i = 0; i < tag->npairs; i++)
	   {
		if (!tag->names[i]) continue;
		if (strcmp(tag->names[i],"TRIES")) continue;
		if (!tag->values[i]) continue;
		cp = strchr(tag->values[i],',');
		if (!cp)
		   {
			tryurl = tag->values[i];
		   }
		else
		   {
			*cp = 0;
			trymax = atoi(tag->values[i]);
			tryurl = cp + 1;
			tryi = 1;
		   }
		break;
	   }
	k = 0; /* timeout time */
	  /* now for timeout */
	for(i = 0; i < tag->ntags; i++)
	   {
		if (!tag->tags[i]->names[0]) continue;
		if (strcmp(tag->tags[i]->names[0],"TIMEOUT")) continue;
		for(j = 0; j < tag->tags[i]->npairs; j++)
		   {
			if (!tag->tags[i]->names[j]) continue;
			if (!strcmp(tag->tags[i]->names[j],"TIME"))
			   {
				if (tag->tags[i]->values[j])
					sscanf(tag->tags[i]->values[j],"%d",&k);
			   }
		   }
	   }
	mipl_flush(MIPL_READ);
	intf = dotextnvoice(tag,0,0,NULL);
	if (intf == 2) return(1);
	  /* now for all the selections */
	for(i = 0; i < tag->ntags; i++)
	   {
		if (!tag->tags[i]->names[0]) continue;
		if (strcmp(tag->tags[i]->names[0],"ASKITEM")) continue;
		for(try = 0;try < trymax;try += tryi)
		   {
			url = 0;
			miplstr_init();
			if (dotextnvoice(tag->tags[i],1,gotv(tag,i),NULL) == 2) return(1);
			  /* output prompt(s) */
			for(j = 0; j < tag->tags[i]->npairs; j++)
			   {
				if (!tag->tags[i]->names[j]) continue;
				if (!tag->tags[i]->values[j]) continue;
				if (!strcmp(tag->tags[i]->names[j],"VOICE"))
					if (tag->tags[i]->values[j])
						if (vok) miplvstr_url(tag->tags[i]->values[j]);
			   }
			if (!mipl_prompt(a,1,intf,k,MIPL_ASK))
			   {
				a[0] = 0;
				  /* now for timeout */
				for(k = 0; k < tag->ntags; k++)
				   {
					if (!tag->tags[k]->names[0]) continue;
					if (strcmp(tag->tags[k]->names[0],"TIMEOUT")) continue;
					if (dotextnvoice(tag->tags[k],0,gotv(tag,k),NULL) == 2) return(1);
					miplstr_init();
					for(j = 0; j < tag->tags[k]->npairs; j++)
					   {
						if (!tag->tags[k]->names[j]) continue;
						if (!strcmp(tag->tags[k]->names[j],"HREF"))
							url = tag->tags[k]->values[j];
						if (!strcmp(tag->tags[k]->names[j],"VOICE"))
							if (tag->tags[k]->values[j]) 
								if (vok) miplvstr_url(tag->tags[k]->values[j]);
					   }
					mipl_out(1);
				   }
				if (url)
				   {
					if (openurl(url,0))
					   {
						sprintf(outbuf,"cannot open/load URL %s\n",url);
						mipl_error(outbuf);
						continue;
					   }
					return(1);
				   }
				continue;
			   }
			/* happy, normal entry */
			if ((!a[0]) || strchr(a,'\033'))  /* if escape */
			   {
				  /* look for escape spec */
				for(k = 0; k < tag->npairs; k++)
				   {
					if (!tag->names[k]) continue;
					if (strcmp(tag->names[k],"ESCAPE")) continue;
					url = tag->values[k];
					if (url)
					   {
						if (openurl(url,0))
						   {
							sprintf(outbuf,"cannot open/load URL %s\n",url);
							mipl_error(outbuf);
							continue;
						   }
						return(1); /* return to execute new url */
					   }
				   }
			   }
			if ((a[0] != 'Y') && (a[0] != 'y') && (a[0] != 'N') &&
				(a[0] != 'n') && (a[0] != '1') && (a[0] != '2')) /* invalid response */
			   {
				  /* find the invalid if any */
				for(k = 0; k < tag->ntags; k++)
				   {
					if (!tag->tags[k]->names[0]) continue;
					if (strcmp(tag->tags[k]->names[0],"INVALID")) continue;
					if (dotextnvoice(tag->tags[k],0,gotv(tag,k),NULL) == 2) return(1);
					miplstr_init();
					for(j = 0; j < tag->tags[k]->npairs; j++)
					   {
						if (!tag->tags[k]->names[j]) continue;
						if (!strcmp(tag->tags[k]->names[j],"HREF"))
							url = tag->tags[k]->values[j];
						if (!strcmp(tag->tags[k]->names[j],"VOICE"))
							if (tag->tags[k]->values[j]) 
								if (vok) miplvstr_url(tag->tags[k]->values[j]);
					   }
					mipl_out(1);
				   }
				if (!url) continue; /* still invalid */
				if (openurl(url,0))
				   {
					sprintf(outbuf,"cannot open/load URL %s\n",url);
					mipl_error(outbuf);
					continue;
				   }
				return(1); /* return to execute new url */
			   }
			  /* if no, dont set pair */
			if ((a[0] == 'n') || (a[0] == 'N') || (a[0] == '2')) break;
			made_selection(tag,tag->tags[i],1);
			break;
		   }
		   /* if too many tries */
		if (try >= trymax)
		   {
			if (!tryurl)
			   {
				printf("maxtries exceeded, but URL not specified");
				mipl_error(outbuf);
				killtemp();
				exit(0);
			   }
			if (openurl(tryurl,0))
			   {
				sprintf(outbuf,"cannot open/load URL %s",tryurl);
				mipl_error(outbuf);
				killtemp();
				return(0);
			   }
			return(1); /* return to execute new url */
		   }
	   }
	return(0);	
}


int dovoicerate(tag)
TAG *tag;
{

	if (tag->values[0]) mipl_voicerate(atoi(tag->values[0]));
	return(0);
}

#ifndef	NORECORD

int dorecord(tag)
TAG *tag;
{
int	i,j,k,r,intf,i0,i1,i2,i3,trymax,maxsec,edit,totime;
char	buf[100],*url,*tryurl,*escurl,*cp;
struct	tm *tm;
FILE	*tfp;


	edit = 1;
	trymax = 0;
	maxsec = 0;
	url = tryurl = escurl = 0;
	totime = 0;
	intf = dotextnvoice(tag,0,0,NULL);
	if (intf == 2) return(1);
 	/* go through the sub tags */
	for(i = 0; i < tag->ntags; i++)
	   {
		if (!tag->tags[i]->names[0]) continue; /* ignore null ones */
		if (!strcmp(tag->tags[i]->names[0],"TIMEOUT"))
		   {
			for(j = 0; j < tag->tags[i]->npairs; j++)
			   {
				if (!tag->tags[i]->names[j]) continue;
				if (!strcmp(tag->tags[i]->names[j],"TIME"))
				   {
					if (tag->tags[i]->values[j])
						sscanf(tag->tags[i]->values[j],"%d",&totime);
				   }
			   }
			continue;
		   }
	   }
	unlink(rectemp);
	  /* go through parameters */
	for(i = 0; i < tag->npairs; i++)
	   {
		if (!tag->names[i]) continue;
		if (!strcmp(tag->names[i],"EDIT"))
		   {
			if (!tag->values[i]) continue;
			if ((*tag->values[i] == 'n') || (*tag->values[i] == 'N'))
				edit = 0;
			continue;
		   }
		if (!strcmp(tag->names[i],"SRC"))
		   {
			if (!tag->values[i]) continue;
			tfp = readurl(tag->values[i],0,0);
			if (!tfp)
			   {
				sprintf(outbuf,"Cannot read URL %s for voice edit!!",tag->values[i]);
				mipl_error(outbuf);
				return(0);
			   }
			fclose(tfp);
			continue;
		   }
		if (!strcmp(tag->names[i],"TIME"))
		   {
			if (!tag->values[i]) continue;
			maxsec = atoi(tag->values[i]);
			continue;
		   }
		if (!strcmp(tag->names[i],"ESCAPE"))
		   {
			if (!tag->values[i]) continue;
			escurl = tag->values[i];
			continue;
		   }
		if (!strcmp(tag->names[i],"TRIES"))
		   {
			if (!tag->values[i]) continue;
			cp = strchr(tag->values[i],',');
			if (!cp)
			   {
				tryurl = tag->values[i];
			   }
			else
			   {
				*cp = 0;
				trymax = atoi(tag->values[i]);
				tryurl = cp + 1;
			   }
			continue;
		   }
	   }
	mipl_flush(MIPL_READ);
	r = mipl_edit(edit,rectemp,maxsec,totime,trymax,RECTEMP,escurl);
	mipl_flush(MIPL_READ);
	if (r == MIPL_TIMEOUT) /* menu timeout */
	   {
		  /* now for timeout */
		for(i = 0; i < tag->ntags; i++)
		   {
			if (!tag->tags[i]->names[0]) continue;
			if (strcmp(tag->tags[i]->names[0],"TIMEOUT")) continue;
			if (dotextnvoice(tag->tags[i],0,gotv(tag,i),NULL) == 2) return(1);
			miplstr_init();
			for(k = 0; j < tag->tags[k]->npairs; k++)
			   {
				if (!tag->tags[k]->names[j]) continue;
				if (!strcmp(tag->tags[k]->names[j],"HREF"))
					url = tag->tags[k]->values[j];
				if (!strcmp(tag->tags[k]->names[j],"VOICE"))
					if (tag->tags[k]->values[j]) 
						miplvstr_url(tag->tags[k]->values[j]);
			   }
			mipl_out(1);
		   }
		if (url)
		   {
			if (openurl(url,0))
			   {
				sprintf(outbuf,"cannot open/load URL %s\n",url);
				mipl_error(outbuf);
				return(0);
			   }
			return(1);
		   }
		return(0);
	   }
	if (r == MIPL_MAXTRIES) /* max tries */
	   {
		if (!tryurl) return(0);
		if (openurl(tryurl,0))
		   {
			sprintf(outbuf,"cannot open/load URL %s",tryurl);
			mipl_error(outbuf);
			return(0);
		   }
		return(1);
	   }
	if (r == MIPL_ESCAPE) /* if escape */
	   {
		if (!escurl) return(0);
		if (openurl(escurl,0))
		   {
			sprintf(outbuf,"cannot open/load URL %s",escurl);
			mipl_error(outbuf);
			return(0);
		   }
		return(1);
	   }
	k = POSTFILE;
	for(j = 0; j < tag->npairs; j++)
	   {
		if (!tag->names[j]) continue;
		if (!strcmp(tag->names[j],"ACTION"))
		   {
			if (tag->values[j]) url = tag->values[j];
		   }
		if (!strcmp(tag->names[j],"METHOD"))
		   {
			if (tag->values[j])
			   {
				if (!strcmp(tag->values[j],"PUT")) k = PUTFILE;
			   }
		   }
	   }
	if (!url)
	   {
		mipl_error("NO ACTION URL specicified in <RECORD>");
		return(0);
	   }
	if (openurl(url,k))
	   {
		sprintf(outbuf,"cannot open/load URL %s",url);
		mipl_error(outbuf);
		return(0);
	   }
	if (r == 2)
	   {
		killtemp();  /* get rida temp files */
		exit(0); /* if hunged up */
	   }
	return(1);
}

#endif

mimecpy(dst,src)
char *dst,*src;
{

	for(;*src;src++)
	   {
		if ((*src == '+') || (*src == '&') || (*src == '?') ||
		    (*src == '%') || (*src < ' ') || (*src > 126))
		   {
			sprintf(dst,"%%%02X",*src);
			dst += 3;
			continue;
		   }
		if (*src == ' ')
		   {
			*dst++ = '+';
			*dst = 0;
			continue;
		   }
		*dst++ = *src;
		*dst = 0;
	   }
	*dst = 0;
	return;
}

int mimelen(src)
char *src;
{
int	i = 0;

	for(;*src;src++)
	   {
		if ((*src == '+') || (*src == '&') || (*src == '?') ||
		    (*src == '%') || (*src < ' ') || (*src > 126))
		   {
			i += 3;
			continue;
		   }
		i += 1;
	   }
	return(i);
}

#ifdef	ISC

char *strstr(s1,s2)
char *s1,*s2;
{
int	i,j;

	i = 0;
	while(*s1)
	   {
		if (*s1++ == s2[i])
		   {
			if (!s2[i + 1])	return(s1 - strlen(s2));
			i++;
		   }
		else
		   {
			i = 0;
		   }
	   }
	return(NULL);
}

#endif

